@@ -1,5 +1,6 @@
 # Make a structure exporter that works with plone 2.0
-from csv import writer
+from csv import writer, QUOTE_MINIMAL
+from xml.dom.minidom import getDOMImplementation
 from ConfigParser import ConfigParser
 from StringIO import StringIO
 import os, shutil
@@ -12,6 +13,7 @@ from Acquisition import aq_inner
 from OFS.SimpleItem import SimpleItem
 from OFS.Image import File
 from DateTime import DateTime
+import time
 
 try:
     from Products.CMFCore.permissions import ManagePortal
@@ -75,25 +77,128 @@ class ContentExporter(UniqueObject, SimpleItem):
         """ Set savepath and log """
         self.savepath = self.get_var_path()
         self.out = []
+        self.format = 'CSV'
+        self.dom = None
 
+    def write_folder_element(self, id, objtype, doc):
+        """ XML folders best gathered by type and listed """
+        if self.format == 'XML':
+            text = doc[0].createTextNode(id)
+            name = objtype.lower()
+            name = name.replace(' ','')
+            child = doc[0].createElement(name)
+            child.appendChild(text)
+            plural = '%ss' % name
+            nodes = doc[0].getElementsByTagName(plural)
+            if nodes:
+                element = nodes[0]
+            else:
+                element = doc[0].createElement(plural)
+            element.appendChild(child)
+            doc[1].appendChild(element)            
+        else:
+            self.write_element(name=id, data=objtype, doc=doc, space=False)
 
+    def write_element(self, name, data, listed=False, doc=None, space=True):
+        """ abstract output writing to allow different formats """
+        if self.format == 'XML':
+            node = None
+            if listed:
+                if name.endswith('s'):
+                    item_name = name[:-1]
+                else:
+                    item_name = 'item'
+                node = doc[0].createElement(item_name)
+                for item in data:
+                    text = doc[0].createTextNode(item)
+                    node.appendChild(text)
+            else:
+                try:
+                    node = doc[0].createTextNode(data)
+                except:
+                    pass
+            if node:
+                element = doc[0].createElement(name)
+                element.appendChild(node)
+                doc[1].appendChild(element)
+        else:
+            if listed:
+                if data:
+                    data = SEPARATOR.join(data)
+                else:
+                    data = ''
+            if space:
+                doc[1].writerow((name, ' %s' % data))
+            else:
+                doc[1].writerow((name, data))
+            return 
+
+    def get_doc(self, delimiter=':'):
+        """ Get doc as xml minidom or csv string stream
+            and its writer as a tuple  
+        """
+        if self.format == 'XML':
+            if delimiter == ',':
+                tag = 'items'
+            else:
+                tag = 'properties'
+            doc = self.dom.createDocument(None, tag, None)
+            node = doc.documentElement
+            return (doc, node)
+        else:
+            ostream = StringIO()
+            return (ostream, writer(ostream,
+                                    delimiter=delimiter,
+                                    quoting=QUOTE_MINIMAL))
+
+    def get_output(self, doc, headfoot=None):
+        """ get output from doc for writing to file
+            Unfortunately csv or xml writers both 
+            require some minor format and header of footer tweaks
+        """
+        if self.format == 'XML':
+            output = doc[0].toprettyxml(indent="  ")
+            if headfoot and headfoot != 'DEFAULT':
+                # Add html at end of xml
+                return output.replace('</properties>',
+                                      '''<html>![CDATA[\n%s]]</html>
+                                         \n</properties>''' % headfoot)
+            else:
+                return output
+        else:
+            if headfoot == 'DEFAULT':
+                output = "[%s]\n%s" % (headfoot, doc[0].getvalue())
+            elif headfoot:
+                output = "%s\n\n%s" % (doc[0].getvalue(), headfoot)                
+            else:
+                output = doc[0].getvalue()
+            #if hasattr(os, 'O_BINARY'):
+            #    output = output.replace('"\r','')
+            #    output = output.replace("\r", '')
+            # kill minimal quotes
+            output = output.replace('\r', '')
+            output = output.replace('"\n', '\n')
+            output = output.replace(':" ', ': ')
+            return output
+        return 'NO FORMAT SUPPLIED'
+        
     def get_var_path(self):
         """ Find var if buildout or old style zope layout or
             test runner where instance home is the buildout-cache!
         """
         var = os.path.join(INSTANCE_HOME, 'var')
         if os.path.exists(var):
-            return os.path.join(var,'structure')
-        for folder in ['parts','buildout-cache']:
+            return os.path.join(var, 'structure')
+        for folder in ['parts', 'buildout-cache']:
             parts = INSTANCE_HOME.find(folder)
             if parts>-1:
                 if os.path.exists(os.path.join(INSTANCE_HOME[0:parts], 'var')):
                     return os.path.join(var,'structure')
                 else:
                     return os.path.join(INSTANCE_HOME[0:parts], 'structure')
-        for var in ['/tmp','\\temp']:
+        for var in ['/tmp', '\\temp']:
             if os.path.exists(var):
-                return os.path.join(var,'structure')
+                return os.path.join(var, 'structure')
 
 
     security.declareProtected( ManagePortal, 'manage_runExport' )
@@ -102,13 +207,20 @@ class ContentExporter(UniqueObject, SimpleItem):
             option to pass in the portal object so this can be run
             more easily by external scripts
         """
-        request = getattr(self,'REQUEST',{})
+        request = getattr(self, 'REQUEST', {})
         if not root:
-            root = request.get('root','')
+            root = request.get('root', '')
+        format = request.get('format', 'CSV')
         if root:
             if not portal:
                 portal = getToolByName(self, 'portal_url').getPortalObject()
-            self.export(root=root, portal=portal, users=request.get('exportusers',None))
+            self.export(root=root,
+                        portal=portal,
+                        users=request.get('exportusers', None),
+                        format = format
+                        )
+        #if self.dom:
+        #    self.dom.unlink()
         if hasattr(request, 'RESPONSE'):
             request.RESPONSE.redirect('manage_export')
 
@@ -117,7 +229,7 @@ class ContentExporter(UniqueObject, SimpleItem):
         """ return the out log file of export actions """
         return self.out 
     
-    def write_file(self, path, filename='', data=''):
+    def write_file(self, path, filename='', data='', modified=None):
         """ Save the file directly to the file system var folder
             If no file or data is supplied this just creates a folder
         """
@@ -138,7 +250,12 @@ class ContentExporter(UniqueObject, SimpleItem):
                 currentpath = folderpath
 
         if filename:
-            filepath=os.path.join(folderpath,filename)
+            if self.format == 'XML':
+                if filename.startswith('.'):
+                    filename = '%s.xml' % filename
+                elif filename.endswith('.html'):
+                    filename = filename.replace('.html', '.xml')
+            filepath=os.path.join(folderpath, filename)
             if os.path.exists(filepath) and os.path.isfile(filepath):
                 os.remove(filepath)
             ofd = None
@@ -177,16 +294,26 @@ class ContentExporter(UniqueObject, SimpleItem):
                                     os.write(ofd, data)
                             except Exception, error:
                                 self.out.append("Sorry failed to write to %s due to %s" % (filepath,error))
+                self.set_times(ofd, modified)
                 os.close(ofd)
         #debug self.out.append('Wrote %s' % filepath)
         return
 
+    def set_times(self, ofd, modified):
+        """ Set the last access and modified time for the file system """
+        if modified:
+            try:
+                times = (time.time(), modified.timeTime())
+                os.utime(ofd, times)
+            except:
+                pass
+        return
+
     def export_users(self, portal):
         """ Write out users and roles in generic setup XML format
             Dump memberdata contents in RFC822 csv format
         """
-        ostream = StringIO()
-        csv_writer = writer(ostream)
+        folderdoc = self.get_doc(delimiter=',')        
         membership = getToolByName(portal, 'portal_membership')
         memberdata = getToolByName(portal, 'portal_memberdata')
         self.write_file('/acl_users','','')
@@ -195,7 +322,9 @@ class ContentExporter(UniqueObject, SimpleItem):
         for user_id in memberdata._members.keys():
             u = membership.getMemberById(user_id)
             if u is not None:
-                csv_writer.writerow((user_id, 'Memberdata'))
+                self.write_element(name=user_id, data='Memberdata',
+                                   doc=folderdoc, space=False)                
+                doc = self.get_doc()
                 try:
                     password = portal.acl_users._user_passwords[user_id]
                 except:
@@ -210,12 +339,15 @@ class ContentExporter(UniqueObject, SimpleItem):
                         'password_hash': password,
                        }
                 user_info.append(info)
-                text = "[DEFAULT]\n"
                 for prop in memberdata.propertyIds():
                     data = u.getProperty(prop)
                     if data:
-                        text += "%s: %s\n" % (prop,data)                    
-                self.write_file(path,user_id,text)
+                        self.write_element(name=prop, data=data, doc=doc)
+                user_output = self.get_output(doc)
+                if self.format == 'XML':
+                    self.write_file(path, '%s.xml' % user_id, user_output)
+                else:
+                    self.write_file(path, user_id, "[DEFAULT]\n%s" % user_output)
                 #TODO: add portait export
                 #obj = memberdata._getPortrait(user_id)
                 #if obj:
@@ -224,24 +356,24 @@ class ContentExporter(UniqueObject, SimpleItem):
         info = {}
         info['title'] = 'source_users'
         info['users'] = user_info
-        template = DTMLFile(os.path.join('xml','zodbusers.xml'),
+        template = DTMLFile(os.path.join('xml', 'zodbusers.xml'),
                             globals()).__of__(portal) 
-        self.write_file('/acl_users','source_users.xml', template(options=info))
-        template = DTMLFile(os.path.join('xml','zodbroles.xml'),
+        self.write_file('/acl_users', 'source_users.xml', template(options=info))
+        template = DTMLFile(os.path.join('xml', 'zodbroles.xml'),
                             globals()).__of__(portal) 
         info = self._getRoleInfo(portal)
-        self.write_file('/acl_users','portal_role_manager.xml',
+        self.write_file('/acl_users', 'portal_role_manager.xml',
                                             template(options=info))
-        template = DTMLFile(os.path.join('xml','zodbgroups.xml'),
+        template = DTMLFile(os.path.join('xml', 'zodbgroups.xml'),
                             globals()).__of__(portal) 
         info = self._getGroupInfo(portal)
-        self.write_file('/acl_users','source_groups.xml',
+        self.write_file('/acl_users', 'source_groups.xml',
                                      template(options=info))
-        output = ostream.getvalue()
+        output = self.get_output(folderdoc)
         # strip Windows line endings
         if hasattr(os, 'O_BINARY'):
-            output = output.replace('\r','')
-        self.write_file(path,'.objects',output)
+            output = output.replace('\r', '')
+        self.write_file(path, '.objects', output)
         self.out.append('Exported member data')
 
     def _getRoleInfo(self, portal):
@@ -320,8 +452,7 @@ class ContentExporter(UniqueObject, SimpleItem):
 
     def write_folder(self, folder, path):
         """ Write the contents of folder out """
-        ostream = StringIO()
-        csv_writer = writer(ostream)
+        folderdoc = self.get_doc(delimiter=',')
         path = path[self.lenportal:]
 
         for id in folder.objectIds():
@@ -331,28 +462,31 @@ class ContentExporter(UniqueObject, SimpleItem):
                 if obj and not str(obj) == '<PythonScript at %s>' % id:
                     try:
                         objtype = obj.getTypeInfo().getId()
-                        objtype = TYPEMAP.get(objtype,objtype)
+                        objtype = TYPEMAP.get(objtype, objtype)
                     except:
                         objtype = None
                     if objtype:
-                        self.export_object(id,obj,objtype,path)
-                        csv_writer.writerow((id, objtype))
-
-        output = ostream.getvalue()
+                        doc = self.get_doc()
+                        self.export_object(id, obj, objtype, path, doc)
+                        self.write_folder_element(id=id, objtype=objtype,
+                                                  doc=folderdoc)
+        output = self.get_output(folderdoc)
         # Strip Windows line endings 
         if hasattr(os, 'O_BINARY'):
-            output = output.replace('\r','')
-        self.write_file(path,'.objects',output)
+            output = output.replace('\r', '')
+        self.write_file(path, '.objects', output)
 
-    def export(self, portal, root='/', users='yes', request={}):
+    def export(self, portal, root='/', users='yes', format='CSV', request={}):
         """ Based on generic setup folder export to structre -
             See Products.GenericSetup.interfaces.IFilesystemExporter
         """
+        self.format = format
+        if format == 'XML':
+            self.dom = getDOMImplementation()
         self.workflow_tool = getToolByName(portal, 'portal_workflow')
         portalname = portal.getId()
         self.lenportal = len(portalname) + 1
         self.out = ['Log started at %s' % DateTime()]
-
         # Give this a request attribute since some methods expect it
         # and it is not available when used via current plone 
         if not hasattr(self, 'REQUEST'):
@@ -380,7 +514,6 @@ class ContentExporter(UniqueObject, SimpleItem):
         # Just return everything then filter for folderish later
         results = catalog(path={'query': root})
         # exportable = portal.contentItems()
-        
         self.out.append('Exporting %s content items to zope/var/structure' % len(results))
         
         if root == '/':    
@@ -392,11 +525,15 @@ class ContentExporter(UniqueObject, SimpleItem):
                 if rootobj:
                     id = rootobj.getId()
                     objtype = rootobj.getTypeInfo().getId()
+                    folderdoc = self.get_doc(delimiter=',')
+                    self.write_folder_element(id=id, objtype=objtype,
+                                              doc=folderdoc)
                     self.write_file('/',
                                     '.objects',
-                                    '%s,%s' % (id, objtype)
+                                    self.get_output(folderdoc)
                                     )
-                    self.export_object(id,rootobj,objtype,'/')
+                    doc = self.get_doc()                    
+                    self.export_object(id, rootobj, objtype, '/', doc)
         for brain in results:
             try:
                 obj = brain.getObject(self.REQUEST)
@@ -411,22 +548,23 @@ class ContentExporter(UniqueObject, SimpleItem):
         return self.out
 
 
-    def export_object(self,id,obj,objtype,path):
+    def export_object(self, id, obj, objtype, path, doc):
         """ export file content to filesystem """
-        text = self.properties_marshall(obj)
+        modified = self.properties_marshall(obj, doc)
         # do required fields
-        for prop,data in PROPS['fixed'].items():
+        for prop, data in PROPS['fixed'].items():
             propname = prop
             prop = PROPS['boolean'].get(prop, prop)
-            if hasattr(obj,prop):
-                if getattr(obj,prop,None):
+            if hasattr(obj, prop):
+                if getattr(obj, prop, None):
                     data = True
                 else:
                     data = False
-            text += "%s: %s\n" % (propname,data)
-        text += self.workflow(obj)
-        archetype = self.at_marshall(obj, path)
-        text += archetype
+            self.write_element(name=propname,
+                               data=data,
+                               doc=doc)
+        self.workflow(obj, doc)
+        self.at_marshall(obj, path, doc)
 
         if hasattr(obj,'data'):
             treatas = 'binary'
@@ -439,32 +577,33 @@ class ContentExporter(UniqueObject, SimpleItem):
             try:
                 value = obj['text']
             except:
-                value = getattr(obj,'text', '')
-            if text:
+                value = getattr(obj, 'text', '')
+            if value:
                 if type(value) not in StringTypes:
                     value = str(value)
-                text += '\n\n%s' % value.replace("\r", "")
-            self.write_file(path,id,text)
+                output = self.get_output(doc, value.replace('\r', ""))
+                self.write_file(path, id, output, modified)
         else:
-            text = "[DEFAULT]\n%s" % text
             if treatas == 'folder':
-                self.write_file(os.path.join(path,id),'.properties',text)
+                self.write_file(os.path.join(path, id), '.properties',
+                                self.get_output(doc, 'DEFAULT'))
             else:
-                self.write_file(path,id + '.ini',text)
-                self.write_file(path,id,obj.data)
+                self.write_file(path, id + '.ini', self.get_output(doc, 'DEFAULT'))
+                self.write_file(path, id, obj.data, modified)
         return
 
-    def workflow(self, obj):
+    def workflow(self, obj, doc):
         """ Get list of workflows and matching states as lines fields """
         chain = self.workflow_tool.getChainFor(obj)
-        text = ''
         if chain:
-            text += "workflows: %s\n" % SEPARATOR.join(chain)
+            self.write_element(name='workflows', data=chain,
+                               listed=True, doc=doc)
             states = []
             for wf_id in chain: 
-                states.append(self.workflow_tool.getInfoFor(obj,'review_state',wf_id))
-            text += "states: %s\n" % SEPARATOR.join(states)
-        return text
+                states.append(self.workflow_tool.getInfoFor(obj, 'review_state', wf_id))
+            self.write_element(name='states', data=states,
+                               listed=True, doc=doc)            
+        return 
 
     def stringify(self, value):
         """ Ensure properties or fields that are files or other types are
@@ -478,12 +617,11 @@ class ContentExporter(UniqueObject, SimpleItem):
             value = value[:-1]
         return value.replace("\n",SEPARATOR)
 
-    def at_marshall(self, obj, path):
+    def at_marshall(self, obj, path, doc):
         """ Check whether object is an archetype and if so
             marshall the fields to properties text and save file field objects"""
         if not hasattr(obj,'Schema'):
             return ''
-        text = ''
         p = obj.getPrimaryField()
         pname = p and p.getName() or None
         fields = obj.Schema().fields()
@@ -499,8 +637,8 @@ class ContentExporter(UniqueObject, SimpleItem):
                 value = None
             #FIXME: check to see if this file is the data file rather than only
             # doing extra files for objects with no data attribute.
-            if f.type in ('file','image'):
-                if not hasattr(obj,'data'):
+            if f.type in ('file', 'image'):
+                if not hasattr(obj, 'data'):
                     filename = '%s.%s' % (obj.getId(), name)
                     self.write_file(path, filename, value)                
                     value = 'EXTRAFILE'
@@ -508,41 +646,47 @@ class ContentExporter(UniqueObject, SimpleItem):
                     value = 'DATAFILE'
             #TODO: Add GSXML style ATReference handling all archetypes in plone 2.1 or later        
             if value != None:
-                text += '%s: ' % name
+                listed = False
                 if type(value) not in StringTypes and type(value) in [ListType, TupleType]:
                     if value:
-                        values = [self.stringify(v) for v in value]
-                        text += SEPARATOR.join(values)
+                        data = [self.stringify(v) for v in value]
+                        listed = True
                 else:
-                    text += self.stringify(value)
-                text += "\n"
-        return text
+                    data = self.stringify(value)
+                self.write_element(name=name, data=data,
+                                   listed=listed, doc=doc)
+        return 
 
-    def properties_marshall(self, obj):
+    def properties_marshall(self, obj, doc):
         """ Pull out dublin core, workflow state and
             other basic plone properties 
         """
-        metadata = ''
-        for prop,method in PROPS['string'].items():
+        modified = None
+        for prop, method in PROPS['string'].items():
             if hasattr(obj,method):
-                data = self.stringify(getattr(obj,method)())
+                data = self.stringify(getattr(obj, method)())
             else:
                 data = ''
-            metadata += "%s: %s\n" % (prop,data)
-        for prop,method in PROPS['date'].items():
-            if hasattr(obj,method):
-                data = str(getattr(obj,method)())
+            self.write_element(name=prop, data=data, doc=doc)
+        for prop, method in PROPS['date'].items():
+            if hasattr(obj, method):
+                data = str(getattr(obj, method)())
             if not data:
                 data = 'None'
-            metadata += "%s: %s\n" % (prop,data)
-        for prop,method in PROPS['list'].items():
-            me = getattr(obj,method,None)
+            else:
+                if prop == 'modification_date':
+                    modified = data
+            self.write_element(name=prop, data=data, doc=doc)        
+        for prop, method in PROPS['list'].items():
+            me = getattr(obj, method, None)
             if me:
-                values = [self.stringify(v) for v in me()]
-                data = SEPARATOR.join(values)
+                data = [self.stringify(v) for v in me()]
+                listed = True 
             else:
                 data = ''
-            metadata += "%s: %s\n" % (prop,data)
+                listed = False
+            self.write_element(name=prop, data=data,
+                                          listed=listed, doc=doc)
         creators = []
         for prop,method in PROPS['user'].items():
             if hasattr(obj,method):
@@ -550,18 +694,20 @@ class ContentExporter(UniqueObject, SimpleItem):
                 creators.append(data)
             else:
                 data = ''
-            metadata += "%s: %s\n" % (prop,data)
+            self.write_element(name=prop, data=data, doc=doc)
         if creators:
             creators = set(creators)
-            metadata += "creators: %s\n" % SEPARATOR.join(creators)
+            self.write_element(name='creators', data=creators,
+                               listed=True, doc=doc)
         objtype = obj.getTypeInfo().getId()
         if objtype in TYPEMAP.keys():
             propmap = NONATPROPS[objtype]
             for prop in propmap.keys():
                 if hasattr(obj,prop):
-                    data = self.stringify(getattr(obj,prop,''))
-                    metadata += "%s: %s\n" % (propmap[prop],data)
-        return metadata
+                    data = self.stringify(getattr(obj, prop, ''))
+                    self.write_element(name=propmap[prop],
+                                       data=data, doc=doc)
+        return modified
         
 InitializeClass(ContentExporter)
 
